pub enum Error {
Timeout,
RpcError(RpcError<TransportErrorKind>),
BlockNotFound,
Closed,
Lagged(u64),
}Expand description
Errors that can occur when using super::RobustProvider.
Variants
Timeout
The operation exceeded the configured timeout.
RpcError(RpcError<TransportErrorKind>)
An RPC error occurred after exhausting all retry attempts.
BlockNotFound
The requested block was not found.
This error is returned when the underlying provider returns None for the requested
block, or when detecting client-specific RPC error responses that indicate a missing block
(e.g., Geth’s error code -32000 with a “block … not found”-like message).
Note: This classification has been verified on Anvil, Reth, and Geth. Other clients
may use different error codes/messages; in those cases the error may surface as
Error::RpcError.
Closed
The subscription channel was closed.
Lagged(u64)