IntoRobustProvider

Trait IntoRobustProvider 

pub trait IntoRobustProvider<N: Network = Ethereum> {
    // Required method
    fn into_robust_provider(
        self,
    ) -> impl Future<Output = Result<RobustProvider<N>, Error>> + Send;
}
Conversion trait for types that can be turned into a RobustProvider.

Required Methods

fn into_robust_provider( self, ) -> impl Future<Output = Result<RobustProvider<N>, Error>> + Send

Convert self into a RobustProvider.

Errors

Returns an error if the primary or any fallback provider fails to connect.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementations on Foreign Types

Source

impl<F, P, N> IntoRobustProvider<N> for FillProvider<F, P, N>
where F: TxFiller<N> + Send + 'static, P: Provider<N> + Send + 'static, N: Network,

Source

impl<N> IntoRobustProvider<N> for DynProvider<N>
where N: Network,

Source

impl<N: Network> IntoRobustProvider<N> for &str

Source

impl<N: Network> IntoRobustProvider<N> for RootProvider<N>

Source

impl<N: Network> IntoRobustProvider<N> for Url

Source

impl<P, N> IntoRobustProvider<N> for CallBatchProvider<P, N>
where P: Provider<N> + Send + 'static, N: Network,

Source

impl<P, N> IntoRobustProvider<N> for CacheProvider<P, N>
where P: Provider<N> + Send + 'static, N: Network,

Implementors